using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace CodeStockDemo
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class ToggleButton : Microsoft.Xna.Framework.DrawableGameComponent
    {
        const float ON_ANGLE = MathHelper.Pi / 6;
        const float OFF_ANGLE = -ON_ANGLE;
        const float RADIANS_PER_SECOND = MathHelper.Pi;

        static Texture2D @base, lever;
        SpriteBatch spriteBatch;
        bool isMovingOn, isMovingOff;
        float angle = OFF_ANGLE;

        public ToggleButton(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        public bool IsChecked { set; get; }

        public Vector2 Location { set; get; }

        public Vector2 Size
        {
            get { return new Vector2(100, 200); }
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(this.GraphicsDevice);

            // Only load the shared bitmaps if they haven't been loaded yet
            if (@base == null)
                @base = this.Game.Content.Load<Texture2D>("base");

            if (lever == null)
                lever = this.Game.Content.Load<Texture2D>("lever");

            base.LoadContent();
        }

        public bool ProcessGesture(GestureSample gesture)
        {
            if (gesture.GestureType == GestureType.Tap)
            {
                Vector2 position = gesture.Position - this.Location;

                if (position.X > 0 && position.X < this.Size.X &&
                    position.Y > 0 && position.Y < this.Size.Y)
                {
                    if (!IsChecked || isMovingOff)
                    {
                        isMovingOff = false;
                        isMovingOn = true;
                    }
                    else if (IsChecked || isMovingOn)
                    {
                        isMovingOn = false;
                        isMovingOff = true;
                    }
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            float increment = RADIANS_PER_SECOND * (float)gameTime.ElapsedGameTime.TotalSeconds;

            if (isMovingOn)
            {
                angle += increment;

                if (angle >= ON_ANGLE)
                {
                    angle = ON_ANGLE;
                    IsChecked = true;
                    isMovingOn = false;
                }
            }
            else if (isMovingOff)
            {
                angle -= increment;

                if (angle <= OFF_ANGLE)
                {
                    angle = OFF_ANGLE;
                    IsChecked = false;
                    isMovingOff = false;
                }
            }

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin();
            spriteBatch.Draw(lever, this.Location + new Vector2(50, 150), null, 
                             IsChecked ? Color.Green : Color.Red,
                             angle, new Vector2(12.5f, 150), 1, SpriteEffects.None, 0);
            spriteBatch.Draw(@base, this.Location + new Vector2(0, 100), Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
